#!/bin/sh

# NAME:
#	scm-funcs.sh - scm wrapper funcs
#
# SYNOPSIS:
#	scm-funcs.sh "op" "args"
#	scm "op" "args"
#	scm-"op" "args"
#
# DESCRIPTION:
#	It can be annoying to keep track of which SCM a given project
#	is using, and while some SCMs like GIT automatically pipe
#	outout through a pager, not all do.
#	This script provides a number of functions to compensate.
#
#	It can be read into a shell, and the functions used directly,
#	or it can be run as a separate process, in which case the "op"
#	is either included in the name or is the first argument.
#
# NOTE:
#	Only the common operations: 'blame' 'diff' 'help' 'log'
#	'status' are provided by default.
#	
#	A simple 'revert' operation is provided for CVS.
#	

# RCSid:
#	$Id: scm-funcs.sh,v 1.23 2024/06/16 20:13:28 sjg Exp $
#
#	@(#) Copyright (c) 2020-2024 Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

$_HAVE_SH . have.sh

is_posix_shell() {
    case "$isPOSIX_SHELL" in
    true) return 0;;
    false) return 1;;
    *)
        case "$local" in
        local) return 0;;
        :)  return 1;;
        *)  if (echo ${PATH%:*}) > /dev/null 2>&1; then
                local=local
                return 0
            else
                local=:
                return 1
            fi
            ;;
        esac
        ;;
    esac
}

get_SCM() {
    clues="CVS/Entries .hg .svn .git"
    for d in . .. ${SRCTOP:-${SB_SRC:-$SB/src}} $SB ...
    do
        for clue in $clues
        do
            : d=$d
            case "$d" in
            ...)
                if type find_it > /dev/null 2>&1; then
                    find_it --start .. --path $clues | sed 's,.*[./],,;s,Entries,cvs,'
                fi
                return
                ;;
            esac
            test -s $d/$clue || continue
            case "$clue" in
            CVS/*) echo cvs; return;;
            .*) echo ${clue#.}; return;;
            esac
        done
    done
}

# tr is insanely non-portable wrt char classes, so we need to
# spell out the alphabet. sed y/// would work too.
toUpper() {
	${TR:-tr} abcdefghijklmnopqrstuvwxyz ABCDEFGHIJKLMNOPQRSTUVWXYZ
}

toLower() {
	${TR:-tr} ABCDEFGHIJKLMNOPQRSTUVWXYZ abcdefghijklmnopqrstuvwxyz
}

# a simple revert method for CVS
cvs_revert() {
    t=/tmp/.$USER.cr$$
    d=$t.d
    
    for p in "$@"
    do
        test -f $p || continue
        ${CVS:-cvs} diff -u $p > $d
        test -s $d || continue
        ${PATCH:-patch} -p0 -R < $d > $t.p 2>&1  &&
        echo Reverted $p
    done
    rm -f $t.*
}
        
scm_op() {
    op=$1; shift
    case "$SCM,$op" in
    cvs,blame) op=annotate;;
    cvs,revert) cvs_revert "$@"; return;;
    esac
    $SCM_CMD $op "$@" | ${PAGER}
}

scm_ops() {
    op=$1; shift
    for f in "$@"
    do
        scm_op $op $f || break
    done
}

# a convenience wrapper
scm() {
    op=$1; shift
    case "$op" in
    commit) $SCM_CMD $op "$@"; return;;
    esac
    for f in scm_$op ${SCM}_$op
    do
        if have $f; then
            $f "$@"
            return
        fi
    done
}

_scm_diff() {
    case "$1" in
    --xargs) _xargs=xargs; shift;;
    *) _xargs=;;
    esac
    diff_opts=
    case "$SCM" in
    cvs) diff_opts="-up";;
    git) diff_opts="--full-index";;
    hg) diff_opts="-p";;
    svn) diff_opts="-x -p";;
    esac
    $_xargs $SCM_CMD diff $diff_opts "$@"
}

diff2list() {
    # do not assume SCM that generated patch
    sed -n \
    -e '/^diff.* -r[1-9][0-9]*\.[1-9]/d' \
    -e '/^diff.*--git/s,.* b/,,p' \
    -e '/^diff/s,.* \(b/\)*,,p' \
    -e '/^Index:/s,Index: ,,p' \
    -e '/^Property changes on:/s,Property changes on: ,,p' \
    "$@" | sort -u
}

PAGER=${PAGER:-more}
SCM=${SB_SCM:-${SCM:-`get_SCM`}}
SCM_VAR=`echo $SCM | toUpper`
eval SCM_CMD=\${$SCM_VAR:-$SCM}

# git & hg do not need PAGER for most things
# but it doesn't hurt and lends consistency
# also allows use of scm_*
for op in blame diff help log status $SCM_OPS
do
    case "$op" in
    diff) _scm_op=_scm_diff;;
    *) _scm_op="$SCM_CMD $op";;
    esac
    eval "scm_$op() { $_scm_op \"\$@\"| ${PAGER}; }"
    eval "${SCM}_$op() { $_scm_op \"\$@\"| ${PAGER}; }"
    case "$op" in
    log) # multiple args handle each with PAGER individually
        eval "scm_${op}s() { scm_ops $op \"\$@\"; }"
        eval "${SCM}_${op}s() { scm_ops $op \"\$@\"; }"
        ;;
    esac
done

case "/$0" in
*/diffl*|*/diff2l*)
    diff2list "$@"
    ;;
*/scm*)
    MYNAME=scm
    if is_posix_shell; then
        Myname=${0##*/}
        Myname=${Myname%.sh}
    else
        Myname=`basename $0 .sh`
    fi
    case "$Myname" in
    scm-funcs|scm) scm "$@";;
    scm-*)
        if is_posix_shell; then
            op=${Myname#scm-}
        else
            op=`expr $Myname : 'scm-\(.*\)'`
        fi
        scm $op "$@"
        ;;
    esac
esac
